@@ -731,6 +731,10 @@ static int bget_swab = 0;
 
 #if PERL_VERSION < 10
 #define BSET_gp_sv(gv, arg)		GvSV((GV*)gv) = arg
+#elif PERL_VERSION >= 21 /* v5.21.7-259-g819b139 2015-01-04 */
+#define BSET_gp_sv(gv, arg)		\
+    isGV_with_GP_on((GV*)gv);		\
+    GvSV((GV*)gv) = arg
 #else
 #define BSET_gp_sv(gv, arg)		\
     isGV_with_GP_on((GV*)gv);		\
@@ -769,29 +773,58 @@ static int bget_swab = 0;
 #define BSET_op_pmflags(r, arg)		r = arg
 
 /* restore dups for stdin, stdout and stderr */
-#define BSET_xio_ifp(sv,fd)						\
-    STMT_START {							\
-      if (fd == 0) {							\
-	IoIFP(sv) = IoOFP(sv) = PerlIO_stdin();				\
-      } else if (fd == 1) {						\
-	IoIFP(sv) = IoOFP(sv) = PerlIO_stdout();			\
-      } else if (fd == 2) {						\
-	IoIFP(sv) = IoOFP(sv) = PerlIO_stderr();			\
-      }									\
-    } STMT_END
+#define BSET_xio_ifp(sv,fd) STMT_START {            \
+    if (fd == 0) {				    \
+      IoIFP(sv) = IoOFP(sv) = PerlIO_stdin();	    \
+    } else if (fd == 1) {                           \
+      IoIFP(sv) = IoOFP(sv) = PerlIO_stdout();      \
+    } else if (fd == 2) {                           \
+      IoIFP(sv) = IoOFP(sv) = PerlIO_stderr();      \
+    }                                               \
+  } STMT_END
 
 #if PERL_VERSION >= 17
-#define BSET_newpadlx(padl, arg)  STMT_START {		\
-	    padl = (SV*)pad_new(arg);			\
-	    BSET_OBJ_STOREX(padl);			\
-	} STMT_END
-#if (PERL_VERSION >= 19) || ( PERL_VERSION == 19 && PERL_SUBVERSION > 3)
-#define BSET_padl_name(padl, pad)                \
-    PadlistARRAY((PADLIST*)padl)[0] = (PAD*)pad; \
-    PadnamelistMAXNAMED((PAD*)pad) = AvFILL((AV*)pad)
+#define BSET_newpadlx(padl, arg)  STMT_START {      \
+    padl = (SV*)pad_new(arg);                       \
+    BSET_OBJ_STOREX(padl);                          \
+  } STMT_END
+#if PERL_VERSION >= 22
+/* no binary names of lexvars */
+#define BSET_newpadnx(pn, pv) STMT_START {              \
+    pn = (SV*)newPADNAMEpvn(pv, strlen(pv));            \
+    BSET_OBJ_STOREX(pn);                                \
+  } STMT_END
+#define BSET_padn_pv(pn, pv) STMT_START {               \
+    PadnamePV((PADNAME*)pn) = pv;                       \
+    PadnameLEN((PADNAME*)pn) = strlen(pv);              \
+  } STMT_END
+#define BSET_newpadnlx(padnl, max)  STMT_START {        \
+    padnl = (SV*)newPADNAMELIST(max);                   \
+    BSET_OBJ_STOREX(padnl);                             \
+  } STMT_END
+/* Beware: PadnamelistMAX == xpadnl_fill (-1) */
+#define BSET_padnl_push(sv, pn)  STMT_START {           \
+    PADNAMELIST* padnl = (PADNAMELIST*)sv;              \
+    SSize_t ix = 1+PadnamelistMAX((PADNAMELIST*)padnl); \
+    padnamelist_store(padnl, ix, (PADNAME*)pn);         \
+    padnl->xpadnl_max = ix;                             \
+  } STMT_END
+#define BSET_unop_aux(op, pv)  STMT_START {           \
+    cUNOP_AUXx(op)->op_aux = ((UNOP_AUX_item *)pv)+1; \
+  } STMT_END
 #else
-#define BSET_padl_name(padl, pad)  PadlistARRAY((PADLIST*)padl)[0] = (PAD*)pad
+#define BSET_newpadnlx(padnl, flags)  STMT_START {      \
+    padnl = (SV*)pad_new(flags);                        \
+    BSET_OBJ_STOREX(padnl);                             \
+  } STMT_END
 #endif
+/* PadlistNAMES broken as lvalue with v5.21.6-197-g0f94cb1,
+   fixed with 5.22.1 and 5.23.0 */
+#if (PERL_VERSION == 22) || ( PERL_VERSION == 21 && PERL_SUBVERSION > 5)
+# undef PadlistNAMES
+# define PadlistNAMES(pl)       *((PADNAMELIST **)PadlistARRAY(pl))
+#endif
+#define BSET_padl_name(padl, pad)  PadlistARRAY((PADLIST*)padl)[0] = (PAD*)pad
 #define BSET_padl_sym(padl, pad)   PadlistARRAY((PADLIST*)padl)[1] = (PAD*)pad
 #define BSET_xcv_name_hek(cv, arg)                                      \
   STMT_START {                                                          \
@@ -802,6 +835,17 @@ static int bget_swab = 0;
   } STMT_END
 #endif
 
+#ifndef OpSIBLING
+#  define OpSIBLING(o)        (o)->op_sibling
+#  define OpSIBLING_set(o, v) (o)->op_sibling = (v)
+#  define OpMAYBESIB_set(o, s, p) OpSIBLING_set(o, s)
+#else
+#  ifndef OpSIBLING_set
+#    define OpSIBLING_set(o, v) OpMORESIB_set((o), (v))
+#  endif
+#endif
+/* sets the sibling or the parent */
+#define BSET_op_sibling(o, v)  OpMAYBESIB_set(o, v, v)
 
 /* NOTE: The bytecode header only sanity-checks the bytecode. If a script cares about
  * what version of Perl it's being called under, it should do a 'use 5.006_001' or
